# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/ESACCI


Description:
This script automatically downloads ESACCI data 
"""

from .LU import main as LU

__all__ = ['LU']

__version__ = '0.1'
